package core.api.controller;

import core.api.models.auth.SimpleStatusResponse;
import core.api.models.timemanager.tasks.TaskEntityInfo;
import core.api.models.timemanager.tasks.TaskFieldInfo;
import core.entities.timemanager.Task;
import core.entities.timemanager.Taskgroup;
import core.services.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api")
public class TaskController {

    private final TaskService taskService;
    private final TaskgroupService taskgroupService;

    public TaskController(@Autowired TaskService taskService, @Autowired TaskgroupService taskgroupService) {
        this.taskService = taskService;
        this.taskgroupService = taskgroupService;
    }

    @GetMapping("/tasks/{taskgroupID}/{status}")
    public ResponseEntity<?> listTasksOfTaskgroup(@PathVariable long taskgroupID, @PathVariable String status) {
        PermissionResult<Taskgroup> taskgroupPermissionResult = taskgroupService.getTaskgroupByIDAndUsername(taskgroupID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(!taskgroupPermissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(taskgroupPermissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }


        if(status.equalsIgnoreCase("all")) {
            return ResponseEntity.ok(taskgroupPermissionResult.getResult().getTasks().stream().map(TaskEntityInfo::new));
        } else if(status.equalsIgnoreCase("overdue")) {
            //List overdue tasks
            return ResponseEntity.ok(new SimpleStatusResponse("success"));
        } else if(status.equalsIgnoreCase("upcoming")) {
            //List upcoming tasks
            return ResponseEntity.ok(new SimpleStatusResponse("success"));
        } else {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }
    }

    @PutMapping("/tasks/{taskgroupID}")
    public ResponseEntity<?> createTask(@PathVariable long taskgroupID, @RequestBody @Valid TaskFieldInfo taskFieldInfo) {
        PermissionResult<Taskgroup> taskgroupPermissionResult = taskgroupService.getTaskgroupByIDAndUsername(taskgroupID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(!taskgroupPermissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(taskgroupPermissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        ServiceResult<Task> creationResult = taskService.createTask(taskgroupPermissionResult.getResult(), taskFieldInfo);
        if(creationResult.getExitCode() == ServiceExitCode.ENTITY_ALREADY_EXIST) {
            return ResponseEntity.status(409).body(new SimpleStatusResponse("failed"));
        } else {
            return ResponseEntity.ok(new TaskEntityInfo(creationResult.getResult()));
        }
    }
}
