import {AfterViewInit, Component, Input, OnChanges, OnInit, SimpleChanges, ViewChild} from '@angular/core';
import {TaskEntityInfo, TaskService} from "../../../api";
import {MatPaginator} from "@angular/material/paginator";
import {MatSort} from "@angular/material/sort";
import {MatTableDataSource} from "@angular/material/table";
import {TaskEditorComponent} from "../task-editor/task-editor.component";
import {TaskEditorData} from "../task-editor/TaskEditorData";
import {MatDialog, MatDialogRef} from "@angular/material/dialog";

@Component({
  selector: 'app-task-dashboard',
  templateUrl: './task-dashboard.component.html',
  styleUrls: ['./task-dashboard.component.css']
})
export class TaskDashboardComponent implements OnChanges{
  ngOnChanges(): void {
    if(this.taskgroupID != undefined) {
      this.taskService.tasksTaskgroupIDStatusGet(this.taskgroupID!, "all").subscribe({
        next: resp => {
          this.datasource = new MatTableDataSource<TaskEntityInfo>(resp);
          this.datasource.paginator = this.paginator!;
          this.datasource.sort = this.sort!;
        }
      })
    }
  }

  @Input("taskgroupID") taskgroupID: number | undefined
  @ViewChild(MatPaginator) paginator: MatPaginator | undefined
  @ViewChild(MatSort) sort: MatSort | undefined

  displayedColumns: string[] = ['status', 'name', 'eta', 'start', 'deadline', 'finished', 'edit', 'delete'];
  datasource: MatTableDataSource<TaskEntityInfo> = new MatTableDataSource<TaskEntityInfo>();

  constructor(private taskService: TaskService,
              private dialog: MatDialog) {
  }


  applyFilter(event: Event) {
    const filterValue = (event.target as HTMLInputElement).value;
    this.datasource.filter = filterValue.trim().toLowerCase();

    if (this.datasource.paginator) {
      this.datasource.paginator.firstPage();
    }
  }

  getStatusOfTask(task: TaskEntityInfo) {
    return "green";
  }

  deleteTask(task: TaskEntityInfo) {
    //todo: implement task delete api call
  }

  editTask(task: TaskEntityInfo) {
    const taskEditorInfo: TaskEditorData = {
      task: task,
      taskgroupID: this.taskgroupID!
    };
    const dialogRef = this.dialog.open(TaskEditorComponent, {data: taskEditorInfo, minWidth: "400px"})

  }
}
